"""Error types"""


class CumulusLibraryError(Exception):
    """
    Package level error
    """


class CumulusLibrarySchemaError(Exception):
    """
    Package level error
    """


class CountsBuilderError(Exception):
    """Basic error for CountsBuilder"""


class StudyManifestParsingError(Exception):
    """Basic error for StudyManifestParser"""
